@@ -61,6 +61,13 @@
 #include "uri.h"
 #include "widgets.h"
 
+/* signal handling */
+#ifndef HAVE_SIGNAL
+#ifndef HAVE_SIGACTION
+#error "This program needs signals to work!"
+#endif
+#endif
+
 
 #define MAX_LINE_SIZE (_POSIX_PATH_MAX*3)
 #define CHILD_FILE_LENGTH 64
@@ -1263,15 +1270,15 @@ static gboolean SubChildTransfer(char *target, char *source)
     if(!(rc & (RW_ERROR_WRITING_TGT | RW_ERROR_OPENING_TGT)))
     {
 	/* put target with original file mode: */
-#if DEBUG
+#ifdef DEBUG
 	  printf("DBG:doing chmod(%s)...",target);fflush(NULL);
 #endif
  	if(chmod(target, s_stat.st_mode) < 0)
 	{
 	    return process_error("chmod", RW_ERROR_STAT_WRITE_TGT);
 	}
-#if DEBUG
-	  printf("ok\n",target);
+#ifdef DEBUG
+	  printf("ok\n");
 #endif
  	
 /*	neither copied nor moved files should have modification
@@ -1281,27 +1288,27 @@ static gboolean SubChildTransfer(char *target, char *source)
   	  times.modtime=s_stat.st_mtime; 
   	  times.actime=s_stat.st_atime;   
 	  /* set the file time stuff to original file time */
-#if DEBUG
+#ifdef DEBUG
 	  printf("DBG:doing utime(%s)...",target);fflush(NULL);
 #endif
   	  if(utime(target,&times)< 0){	
 	    return process_error("utime", RW_ERROR_WRITING_SRC);
 	  }	  
-#if DEBUG
-	  printf("ok\n",target);
+#ifdef DEBUG
+	  printf("ok\n");
 #endif
 	}
 	
 	if (child_mode & TR_MOVE) {
 	  /* if all worked, try removing original file */
-#if DEBUG
+#ifdef DEBUG
 	  printf("DBG:doing unlink(%s)...",source);fflush(NULL);
 #endif
 	  if(unlink(source) < 0){
 	    return process_error("rm", RW_ERROR_WRITING_SRC);
 	  }
-#if DEBUG
-	  printf("ok\n",target);
+#ifdef DEBUG
+	  printf("ok\n");
 #endif
 	}
 
@@ -1326,6 +1333,9 @@ static void ChildTransfer(gpointer data)
 
     all_recursive = FALSE;
     I_am_child = TRUE;
+#ifdef HAVE_SIGNAL
+    signal(SIGTERM,finish);
+#else 
     {
 	struct sigaction act;
 	act.sa_handler = finish;
@@ -1337,6 +1347,7 @@ static void ChildTransfer(gpointer data)
 #endif
 	sigaction(SIGTERM,&act,NULL);
     }
+#endif
     incomplete_target = FALSE;
     line = (char *)malloc(MAX_LINE_SIZE);
     if(!line)
@@ -1550,7 +1561,9 @@ static int rwStderr(int n, void *data)
 static void cpyForkCleanup(void)
 {
     /* break inner gtk_main() loop */
-    /*fprintf(stderr,"dbg:call to innerloop from forkcleanup\n"); */
+#ifdef DEBUG
+    fprintf(stderr,"dbg:call to innerloop from forkcleanup\n"); 
+#endif
     set_innerloop(FALSE);
     /* unlink(fork_target); */
 }
@@ -1576,8 +1589,9 @@ static int rwStdout(int n, void *data)
 	return TRUE;
     }
 
-
-    /*printf("dbg(rwStdout):%s",line);fflush(NULL); */
+#ifdef DEBUG
+    printf("dbg(rwStdout):%s",line);fflush(NULL); 
+#endif
 
     /* only process child specific lines: */
     if(strncmp(line, "child:", strlen("child:")) != 0)
@@ -1637,7 +1651,9 @@ static int rwStdout(int n, void *data)
      * should continue with a diagnostics message.  */
     strtok(line, ":");
     texto = strtok(NULL, "\n");
-    /*printf("dbg(rwStdout error):%s\n",texto);fflush(NULL); */
+#ifdef DEBUG
+    printf("dbg(rwStdout error):%s\n",texto);fflush(NULL); 
+#endif
 
     print_diagnostics(cpy_treeview, "xf_ERROR_ICON", texto, "\n", NULL);
     process_pending_gtk();
@@ -1662,7 +1678,9 @@ static void rwForkOver(pid_t pid)
     char secs[1024];
     tree_details_t *tree_details = get_tree_details(cpy_treeview);
     rw_fork_obj = NULL;
-    /*fprintf(stderr,"dbg: call to innerloop from forkover()\n");fflush(NULL); */
+#ifdef DEBUG
+    fprintf(stderr,"dbg: call to innerloop from forkover()\n");fflush(NULL); 
+#endif
     set_innerloop(FALSE);
     set_progress(cpy_treeview, -1, -1);
     sprintf(secs, "%d/%d %s %ld %s", nitems, countT, _("files copied in"), (long)(time(NULL) - initial), (time(NULL) - initial == 1) ? _("second") : _("seconds"));
