@@ -37,6 +37,8 @@
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
 
+#include <libxfce4util/util.h>
+
 #include "glade_callbacks.h"
 #include "glade_gui.h"
 #include "glade_support.h"
@@ -159,6 +161,7 @@ static void find_over(void)
 
     /* scroll is in callbacks.c, open_dir(), 
      * but only called if preference enabled */
+    gdk_flush();
     gtk_tree_view_scroll_to_cell(find_treeview, treepath, NULL, TRUE, 0.0, 0.0);
     gtk_tree_path_free(treepath);
     if(!findCount)
@@ -823,8 +826,13 @@ static void on_find_clicked(GtkButton * button, gpointer data)
 		    (void *)argument, 
 		    fork_finished_function, 
 		    FALSE, 
-		    operate_stdout, 
-		    operate_stderr);
+		    operate_stdout, operate_stderr);
+    if (!tree_details->tubo_object){
+            gtk_widget_destroy(lookup_widget(w, "find_dialog"));
+	    print_diagnostics(treeview,"xf_ERROR_ICON","Could not fork!\n",
+			    NULL);
+	    return;
+    }
     g_timeout_add_full(0,260, (GtkFunction) watch_stop, (gpointer) treeview,NULL);
     Gpid = TuboPID(tree_details->tubo_object);
 
