@@ -102,6 +102,8 @@
 #include "misc.h"
 #include "refresh.h"
 #include "tubo.h"
+#include "xfstab.h"
+#include "icons.h"
 
 
 static gdir_t fstat_gdir;
@@ -270,13 +272,20 @@ int is_mounted(char *mnt_point){
 int
 fstab_mount(GtkTreeView *treeview, char *mnt_point, gboolean umnt)
 {
-    gchar *argument[5];
+    gchar *argument[6];
     gchar **ap;
     int i;
     tree_details_t *tree_details = get_tree_details(treeview);
 
     ap = argument;
 
+    if (getenv("XFFM_USE_SUDO") && strlen(getenv("XFFM_USE_SUDO"))){
+	      /* FIXME: do a check whether if the command is sudo allowed
+	       * without password, if so then: */
+	    *ap++ = "sudo";
+    }
+		    
+
     i=is_mounted(mnt_point);
     if (i >= 0){
       if (i) *ap++ = "umount";
