@@ -48,6 +48,7 @@
 #include "misc.h"
 #include "pasteboard.h"
 #include "settings.h"
+#include "xfstab.h"
 
 #define D(x)
 
@@ -784,6 +785,9 @@ void update_icon(GtkTreeView * treeview, GtkTreeIter * iterator)
     }
     return;
 }
+/* defined in src/glade_support.c, but not exported */
+extern void clear_pixmap_directory (void);
+
 /*#define DEBUG*/
 void create_icons(tree_details_t * tree_details)
 {
@@ -823,7 +827,7 @@ void create_icons(tree_details_t * tree_details)
     icon_factory = gtk_icon_factory_new();
     icon_hash = g_hash_table_new(g_str_hash, g_str_equal);
 
-   
+    /* defined in src/glade_support.c, but not exported */
     clear_pixmap_directory();
     
     icon_dir=g_strconcat(PACKAGE_DATA_DIR,"/",PACKAGE,"/pixmaps",NULL);
