@@ -76,6 +76,7 @@
 #include "scramble.h"
 #include "symlink.h"
 #include "widgets.h"
+#include "bookmarks.h"
 
 #define PRINT_STATUS_INPUT print_status(treeview, "xf_QUESTION_ICON", _("Input requested"), NULL)
 #define PRINT_STATUS_DONE print_status(treeview, "xf_INFO_ICON", _("Command done"), NULL)
@@ -83,6 +84,7 @@
 gchar *workdir=NULL;
 extern gchar *bookfile;
 static GList *list;
+extern gchar *active_dbh_file;
 
 #define CURRENT_WORKDIR_HISTORY "xffm%cxffm.workdir.2.dbh"
 static void save_workdir_history(char *p){
@@ -97,7 +99,6 @@ static GList *set_workdir_combo(GtkTreeView * treeview)
     char fname[_POSIX_PATH_MAX];
     tree_details_t *tree_details = get_tree_details(treeview);
     GtkWidget *w;
-    int i;
  
     xfce_get_userfile_r(fname, _POSIX_PATH_MAX-1, CURRENT_WORKDIR_HISTORY,
 		    G_DIR_SEPARATOR);
@@ -133,26 +134,48 @@ static gint on_key_press(GtkWidget * entry, GdkEventKey * event, gpointer data)
     }
     return (FALSE);
 }
+
+
+/*summary:
+ * control-tab moves to next slash or end of string
+ * tab cycles to the next history element that matches
+ * control-del removes the item from the history
+ * */
+
 static gint on_key_press2(GtkWidget * entry, GdkEventKey * event, gpointer data)
 {
     void ok_input(GtkTreeView * treeview);
     gchar *text[2];
     gchar c[]={0,0};
-    gchar *fulltext;
-    GList *tmp;
+    gchar *fulltext=NULL;
+    GList *tmp=list;
     GtkTreeView *treeview = (GtkTreeView *) data;
     GtkEditable *editable=(GtkEditable *)entry;
-    gint pos2,pos=gtk_editable_get_position(editable);
-   /* printf("DBG(2):got key= 0x%x\n",event->keyval);*/
+    gint pos2,pos;
+    gboolean preselection=
+	    gtk_editable_get_selection_bounds (editable,&pos,&pos2);
     
- if (event->keyval == GDK_BackSpace) {
-	if (event->state&GDK_CONTROL_MASK){
-	    gtk_editable_delete_text (editable,0,-1);
-	    return TRUE;
-	}
+    
+    if (!preselection){
+	 pos=gtk_editable_get_position(editable);
+    }	 
+
+    /*printf("DBG(2):pos= %d\n",pos);*/
+   /* printf("DBG(2):got key= 0x%x\n",event->keyval);*/
+
+    /* control tab will move to next slash, 
+     * normal tab will complete on basis
+     * of next candidate in list */
+    
+ if (event->keyval == GDK_BackSpace && (event->state&GDK_CONTROL_MASK)){
+	gtk_editable_delete_text (editable,0,-1);
+	return TRUE;
  }
+
  if (event->keyval == GDK_Tab) {
-	gint start,end,i;
+    gint start,end;
+    if (event->state&GDK_CONTROL_MASK){
+	
     	if (gtk_editable_get_selection_bounds(editable,&start,&end)){
 	/*printf("selection at %d %d\n",start,end);*/
 	  for (;start<=end;start++){
@@ -171,27 +194,80 @@ static gint on_key_press2(GtkWidget * entry, GdkEventKey * event, gpointer data)
 	  return(TRUE);
 	  
 	}
-    
-   
-    return TRUE;
+        return TRUE;
+    }
+    else
+    {
+    	fulltext=gtk_editable_get_chars (editable,0,-1);
+        for (;tmp;tmp=tmp->next){
+	   gchar *p=(gchar *)tmp->data;
+	   if (strcmp(p,fulltext)==0){
+		   tmp=tmp->next;
+		   break;
+	   }
+	}
+	g_free(fulltext);
+    	fulltext=gtk_editable_get_chars (editable,0,pos);
+    }
  }
 
- if ((event->keyval >= GDK_A && event->keyval <= GDK_Z) || 
-	    (event->keyval >= GDK_a && event->keyval <= GDK_z) ){
-	 if (event->keyval >= GDK_a && event->keyval <= GDK_z)
-		*c=event->keyval - GDK_a + 'a';
-	 else
-		*c=event->keyval - GDK_A + 'A';
-	    
-    /* get entry text */
-    text[0]=gtk_editable_get_chars (editable,0,pos);
-    text[1]=gtk_editable_get_chars (editable,pos+1,-1);
-    fulltext=g_strconcat(text[0],c,text[1],NULL);
-    g_free(text[0]);
-    g_free(text[1]);
 
+ /* construct search string... (limited to 128 bytes)*/
+
+ /* control-delete will remove the selected item from the
+  * history dbh file (to remove stale history items) */
+
+ else if (event->keyval == GDK_BackSpace ||
+     event->keyval == GDK_Delete ||
+     event->keyval == GDK_KP_Delete ||
+     (event->keyval >= GDK_KP_0 && event->keyval <= GDK_KP_9) ||
+     (event->keyval >= GDK_space && event->keyval <= GDK_asciitilde)) {
+	/* get entry text */
+
+   	if (event->keyval == GDK_BackSpace ){
+      	  text[0]=gtk_editable_get_chars (editable,0,pos-1);
+    	  text[1]=gtk_editable_get_chars (editable,pos,-1);
+	  if (preselection) {
+		  g_free(fulltext);
+		  fulltext=NULL;
+	  }
+	  else fulltext=g_strconcat(text[0],text[1],NULL);
+	} 
+	else if (event->keyval == GDK_Delete ||
+     	    event->keyval == GDK_KP_Delete){
+	  if (active_dbh_file && event->state&GDK_CONTROL_MASK){ /* remove stale entries */
+    	    fulltext=gtk_editable_get_chars (editable,0,-1);
+	    remove_from_history(active_dbh_file,fulltext);
+	    print_diagnostics(treeview,"xf_INFO_ICON",
+			    _("Variable")," \"",fulltext,"\" ",
+			    _("Cancelled"),"\n",NULL);
+	    g_free(fulltext);
+	    cancel_input(treeview);
+	    return TRUE;
+	  } else {
+      	    text[0]=gtk_editable_get_chars (editable,0,pos);
+    	    text[1]=gtk_editable_get_chars (editable,pos+1,-1);
+	    if (preselection) {
+		  g_free(fulltext);
+		  fulltext=NULL;
+	    }
+	    else fulltext=g_strconcat(text[0],text[1],NULL);
+	  }
+	} else {
+      	  text[0]=gtk_editable_get_chars (editable,0,pos);
+    	  text[1]=gtk_editable_get_chars (editable,pos+1,-1);
+	  *c=event->keyval;	
+	  if (preselection) fulltext=g_strconcat(text[0],c,NULL);
+	  else fulltext=g_strconcat(text[0],c,text[1],NULL);
+	}		
+    	g_free(text[0]);
+    	g_free(text[1]);
+ }
+	 
+ if (fulltext){
     /* look for in ordered GList*/
-    for (tmp=list;tmp;tmp=tmp->next){
+    /*printf("DBG:fulltext is %s\n",fulltext);*/
+    for (;tmp;tmp=tmp->next){
 	gchar *p=(gchar *)tmp->data;
 	if (!p) continue;
 	if (strncmp(fulltext,p,strlen(fulltext))==0){
@@ -206,13 +282,16 @@ static gint on_key_press2(GtkWidget * entry, GdkEventKey * event, gpointer data)
 	    gtk_editable_insert_text (editable,(const gchar *)to_write,
 			     strlen(to_write),&pos2);
     	    gtk_editable_select_region (GTK_EDITABLE (entry), pos, -1);
+    	    g_free(fulltext);
 
 	    return TRUE;
 	}
     }
+    g_free(fulltext);
  }
+ if (event->keyval == GDK_Tab) return TRUE;
 			    
-    return (FALSE);
+ return (FALSE);
 }
 
 
