@@ -34,6 +34,8 @@
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
 
+#include <libxfce4util/util.h>
+
 #include "glade_support.h"
 
 #include "constants.h"
@@ -48,6 +50,7 @@
 #include "reg.h"
 #include "widgets.h"
 #include "xfstab.h"
+#include "bookmarks.h"
 
 extern gchar *bookfile;
 
@@ -62,6 +65,16 @@ extern gchar *bookfile;
  *  {char *extension,char *command,
  *  char *menu_label,char *input_query_text, gboolean queued} 
  * */
+#if 1
+/* here the extension is appendend to "$command $filename$extension $filename". if NULL, 
+ * then  "$command $filename" is used */
+autotype_t autotype_dir[]= {
+	/* archive file extractions (queued) */
+	{".tar.gz","tar -czf ","tar -czf",N_("Output dir"),1},
+	{".iso","mkisofs -dDlLJNRTUP Created_by_XFce4_desktop -o ","mkisofs",N_("Output dir"),1},
+	{NULL,NULL}
+};
+#endif
 
 autotype_t autotype[]= {
 	/* archive file extractions (queued) */
@@ -75,20 +88,32 @@ autotype_t autotype[]= {
 	{".zip","unzip",N_("Uncompress"),NULL,1},
 	
 	{".tgz","tar -xzf ",N_("Extract files from"),N_("Extract into"),1},
+#if 1
+	{".iso","burncd -f /dev/acd0c -s max data ","burncd",NULL,1},
+	{".iso","cdrecord -v speed=24 dev=0,0 ","cdrecord",NULL,1},
+#endif
+	
 	/* FreeBSD package management (queued) */
+#ifndef __linux__
 #ifdef	__FreeBSD__
 	{".tgz","pkg_info -dV ",N_("Package information"),NULL,1},
+	{".tbz","pkg_info -dV ",N_("Package information"),NULL,1},
 #else
 	{".tgz","pkg_info -d ",N_("Package information"),NULL,1},
+	{".tbz","pkg_info -d ",N_("Package information"),NULL,1},
 #endif	/* !__FreeBSD__ */
 	{".tgz","pkg_info -r ",N_("Package requirements"),NULL,1},
 	{".tgz","pkg_info -L ",N_("Package contents"),NULL,1},
-
 	{".tgz","pkg_update ",N_("Update package"),NULL,1},
 	{".tgz","pkg_add ",N_("Install package"),NULL,1},
 	{".tgz","pkg_add -f ",N_("Install package (force)"),NULL,1},
 	
-	
+	{".tbz","pkg_info -r ",N_("Package requirements"),NULL,1},
+	{".tbz","pkg_info -L ",N_("Package contents"),NULL,1},
+	{".tbz","pkg_update ",N_("Update package"),NULL,1},
+	{".tbz","pkg_add ",N_("Install package"),NULL,1},
+	{".tbz","pkg_add -f ",N_("Install package (force)"),NULL,1},
+#endif
 #ifdef __linux__
 	/* RedHat package management (queued) */
 	{".rpm","rpm -q -i -p ",N_("Package information"),NULL,1},
@@ -172,6 +197,34 @@ char *auto_CC_name[]={
    "autotype_CC10",
    NULL
 };
+char *auto_D_name[]={
+   "autotype_D1",
+   "autotype_D2",
+   "autotype_D3",
+   "autotype_D4",
+   "autotype_D5",
+   "autotype_D6",
+   "autotype_D7",
+   "autotype_D8",
+   "autotype_D9",
+   "autotype_D10",
+   NULL
+};
+/* Use as many different names for as many different
+ * autotype menu entries for one file extension type. */
+char *auto_DD_name[]={
+   "autotype_DD1",
+   "autotype_DD2",
+   "autotype_DD3",
+   "autotype_DD4",
+   "autotype_DD5",
+   "autotype_DD6",
+   "autotype_DD7",
+   "autotype_DD8",
+   "autotype_DD9",
+   "autotype_DD10",
+   NULL
+};
 
 static gboolean invalid_input;
 static gboolean local_stuff;
@@ -185,9 +238,9 @@ static void check_select(GtkTreeModel * model, GtkTreePath * path, GtkTreeIter *
 {
     GtkTreeView *treeview = (GtkTreeView *) data;
     tree_details_t *tree_details = get_tree_details(treeview);
-    GtkTreeModel *treemodel = gtk_tree_view_get_model(treeview);
+/*    GtkTreeModel *treemodel = gtk_tree_view_get_model(treeview);*/
     tree_entry_t *en = get_entry(treeview, iter);
-    GtkTreeRowReference *reference;
+/*    GtkTreeRowReference *reference;*/
 
     if(!en || IS_DUMMY_TYPE(en->type)){		    
          GtkTreeRowReference *reference=
@@ -380,8 +433,10 @@ void do_select_popup(GtkTreeView * treeview, GtkTreePath *selectpath,gint32 t)
 			showit(popup, "open1");
 			for (i=0;auto_C_name[i];i++)
 				hideit(popup,auto_C_name[i]);
+			for (i=0;auto_D_name[i];i++)
+				hideit(popup,auto_D_name[i]);
 			hideit(popup, "autotype_Prun");
-			hideit(popup, "autotype_D");
+			hideit(popup, "autotype_D");/* FIXME: these should be removed from glade gui*/
 			hideit(popup, "autotype_R");
 			showit(popup, "open_with_1");
 		}
@@ -458,10 +513,39 @@ void do_select_popup(GtkTreeView * treeview, GtkTreePath *selectpath,gint32 t)
 		    showit(popup, "collect_trash1");
 		    showit(popup, "open_in_xffm1");
 		    showit(popup, "open_in_terminal1");
-		    showit(popup, "autotype_D");
+		    /*showit(popup, "autotype_D");*/
 		    hideit(popup, "print3");
 		    hideit(popup, "touch2");
+		    hideit(popup, "separador10");
 		    hideit(popup, "salir2");
+#if 1
+		{/* autotypes_D */	      
+      		   char *loc;
+		   int i,j;
+		   GtkWidget *label;
+		   for (j=i=0;auto_D_name[j]&&autotype_dir[i].label;i++){
+		       GtkWidget *a=lookup_widget(popup,auto_D_name[j]);
+		       {
+		           gchar *text,*prg;
+    		  	   label = gtk_bin_get_child(GTK_BIN(a));
+		           text=g_strconcat(_("Run")," ",_(autotype_dir[i].label),
+					   " ", FILENAME(en),
+					   NULL);
+			   gtk_label_set_text((GtkLabel *)label,
+					   my_utf_string(text));
+			   g_free(text); text=NULL;
+			   prg=g_strdup(autotype_dir[i].command);
+			   if (strstr(prg," ")) prg=strtok(prg," ");
+			   text=g_find_program_in_path(prg);
+		  	   if (text) showit(popup, auto_D_name[j]);
+			   g_free(text);
+			   g_free(prg);
+			   j++;
+		       }
+		   }
+		}
+#endif
+#if 0
 		    {
         		GtkWidget *label;
         		gchar *text=NULL;
@@ -474,6 +558,7 @@ void do_select_popup(GtkTreeView * treeview, GtkTreePath *selectpath,gint32 t)
 			gtk_label_set_text(GTK_LABEL(label),my_utf_string(text));
 			g_free(text);
 		    }		
+#endif
 		}
 		else if (IS_FILE(en->type))
 		{
@@ -496,6 +581,7 @@ void do_select_popup(GtkTreeView * treeview, GtkTreePath *selectpath,gint32 t)
 		    hideit(popup, "open_in_xffm1");
 		    hideit(popup, "open_in_terminal1");
 		    hideit(popup, "paste1");
+		    hideit(popup, "separador10");
 		    hideit(popup, "salir2");
 		}
 	    }
