@@ -65,7 +65,9 @@
 /****************/
 
 extern GtkWidget *autotype_C;
+extern GtkWidget *autotype_D;
 extern autotype_t autotype[];
+extern autotype_t autotype_dir[];
 extern gchar *workdir;
 extern gchar *xffm_argv0;
 extern gchar *xffm_argv1;
@@ -219,6 +221,7 @@ void on_activate_branch(GtkMenuItem * menuitem, gpointer user_data){
       GtkTreePath *treepath=gtk_tree_model_get_path(treemodel, &iter);
       GtkTreeSelection *selection = gtk_tree_view_get_selection(treeview);
       gtk_tree_view_expand_row(treeview, treepath, FALSE);  
+      gdk_flush();
       gtk_tree_view_scroll_to_cell(treeview, treepath, NULL, TRUE, 0.0, 0.0);
       gtk_tree_selection_select_path (selection,treepath);
       gtk_tree_view_set_cursor (treeview,treepath,NULL,FALSE);
@@ -236,7 +239,6 @@ void on_close_activate(GtkMenuItem * menuitem, gpointer user_data)
     
 #if defined(HAVE_GETMNTENT) || defined(HAVE_GETFSENT) || defined(HAVE_GETVFSENT)
     if (strstr(xffm_argv0,"xffstab") && xffm_argv1) {
-	int status;
 	int fstab_mount(GtkTreeView *treeview, char *mnt_point, gboolean umnt);
 	int is_mounted(char *mnt_point);
 	/* if argv[1] was already mounted, this will unmount it too.*/
@@ -414,7 +416,22 @@ on_autotype_C                          (GtkMenuItem     *menuitem,
   j=0;
   if (strstr(autotype[i+offset].command," ")){
       if (strlen(autotype[i+offset].command)>254) return;
-      strcpy(command,autotype[i+offset].command);	      
+      strcpy(command,autotype[i+offset].command);
+      /* sudo-izable stuff */
+      if(getenv("XFFM_USE_SUDO")&&strlen(getenv("XFFM_USE_SUDO"))&&
+	 (strncmp(command,"pkg_add",strlen("pkg_add"))==0 ||
+	  strncmp(command,"pkg_update",strlen("pkg_update"))==0 ||
+	  strncmp(command,"burncd",strlen("rpm"))==0 ||
+	  strncmp(command,"cdrecord",strlen("rpm"))==0 ||
+	  strncmp(command,"rpm",strlen("rpm"))==0 ||
+	  strncmp(command,"dpkg",strlen("dpkg"))==0 )
+	) {
+	      /* FIXME: do a check whether if the command is sudo allowed
+	       * without password, if so then: */
+	      argv[j++]="sudo";
+	      /* when check is implemented, add env var to mcs manager
+	       * (put check also in fstab.c) */
+      }	      
       argv[j++]=strtok(command," ");
       do {
 	      argv[j]=strtok(NULL," ");
@@ -424,6 +441,8 @@ on_autotype_C                          (GtkMenuItem     *menuitem,
       } while (1);
   } else argv[j++]=autotype[i+offset].command;
   argv[j++]=en->path;
+  if (strcmp(argv[0],"burncd")==0 || strcmp(argv[1],"burncd")==0) 
+	  argv[j++]="fixate";
   argv[j]=0;
 
   /* this function call will enter a gtk loop */ 
@@ -443,6 +462,75 @@ on_autotype_C                          (GtkMenuItem     *menuitem,
 void
 on_autotype_D                          (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
+{
+  char *argv[64];
+  char command[255];
+  char output[255];
+  int offset=(int)((long)user_data);
+  char *loc;
+  int i=0,j; 
+  tree_entry_t *en;
+    GtkTreeView *treeview = get_treeview((GtkWidget *) menuitem);
+    tree_details_t *tree_details = (tree_details_t *) get_tree_details(treeview);
+    GtkTreeIter iter;
+    en = get_selected_entry(treeview, &iter);
+
+       
+  /*printf("DBG:offset=%d\n",offset);*/
+  if (!en || !en->path || !IS_DIR(en->type)) return;
+  
+  for (j=0;j<=offset;j++){
+  	/*printf("DBG:command=%s\n",autotype[i+j].command);*/
+  	if (autotype_dir[i+j].command==NULL) return;
+  }
+
+  
+  j=0;
+  if (strstr(autotype_dir[i+offset].command," ")){
+      if (strlen(autotype_dir[i+offset].command)>254) return;
+      strcpy(command,autotype_dir[i+offset].command);
+#if 0
+      /* sudo-izable stuff */
+      if(getenv("XFFM_USE_SUDO")&&strlen(getenv("XFFM_USE_SUDO"))&&
+	 (strncmp(command,"xx",strlen("xx"))==0 ||
+	  strncmp(command,"wahtever",strlen("wthatever"))==0 )
+	) {
+	      /* FIXME: do a check whether if the command is sudo allowed
+	       * without password, if so then: */
+	      argv[j++]="sudo";
+	      /* when check is implemented, add env var to mcs manager
+	       * (put check also in fstab.c) */
+      }	  
+#endif    
+      argv[j++]=strtok(command," ");
+
+      do {
+	      argv[j]=strtok(NULL," ");
+	      if (!argv[j]) break;
+	      j++;
+	      if (j>=64) { argv[63]=0; break; }
+      } while (1);
+  } else argv[j++]=autotype_dir[i+offset].command;
+  if (autotype_dir[i+offset].extension) {
+	  if (strlen(FILENAME(en))+strlen(autotype_dir[i+offset].extension) >254) 
+		  return;
+	  sprintf(output,"%s%s",FILENAME(en),autotype_dir[i+offset].extension);
+	  argv[j++]=output;
+  }
+  argv[j++]=en->path;
+  argv[j]=0;
+
+  /* this function call will enter a gtk loop */ 
+  if (!autofunction_workdir (treeview, en,autotype_dir[i+offset].querypath)) {
+	  return;
+  }
+  /*printf("DBG:");for (j=0;argv[j];j++) printf("%s ",argv[j]);printf("\n");*/
+  if (autotype_dir[i+offset].queued)
+	  autofunction(treeview,en,argv,workdir, autotype_dir[i+offset].queued);
+  else runv(treeview,argv);
+  return;
+}
+#if 0
 {
   char *arguments[5];
   gchar *d;
@@ -473,7 +561,7 @@ on_autotype_D                          (GtkMenuItem     *menuitem,
   g_free(arguments[2]);
   return;
 }
-
+#endif
 
 
 void
