@@ -91,7 +91,7 @@ extern gboolean double_treeview;
 static gboolean correct_selection;
 static void query_selection(GtkTreeModel * model, GtkTreePath * path, GtkTreeIter * iter, gpointer data)
 {
-    GtkTreeView *treeview = (GtkTreeView *) data;
+/*    GtkTreeView *treeview = (GtkTreeView *) data;*/
     if(correct_selection) return;
     correct_selection=TRUE;
     return;
@@ -349,7 +349,6 @@ new_root:
 gboolean find_root(GtkTreeView * treeview, int which)
 {
     GtkTreeModel *treemodel = gtk_tree_view_get_model(treeview);
-    const gchar *g;
     GtkTreeIter iter;
     tree_entry_t *en;
     if(!gtk_tree_model_get_iter_first(treemodel, &iter)) return FALSE;
@@ -722,7 +721,6 @@ gchar *utf_2_local_string(char *t){
     static gchar *s = NULL;
     GError *error = NULL;
     gsize r_bytes, w_bytes;
-    unsigned char *c;
     const char *fc;
     gchar *from_codeset=NULL;
 
@@ -1061,12 +1059,17 @@ static void clean_history_list(GList **list){
  * if top=="", the top entry is the one with the greatest access time for last hit
  * if top=="anything", the entry is set to "anything"
  * */
+gchar *active_dbh_file=NULL;
+
 void get_history_list(GList **in_list,char *dbh_file,char *top){
    DBHashTable *d;
    GList *tmp;
    char *first=NULL;
    time_t last_hit=0;
    the_list=in_list;
+   g_free(active_dbh_file);
+   active_dbh_file=g_strdup(dbh_file);
+	   
    clean_history_list(the_list);	
    if ((d=DBH_open(dbh_file))!=NULL){
 	DBH_foreach_sweep(d,history_mklist);	
@@ -1127,3 +1130,25 @@ void save_to_history(char *dbh_file,char *path2save){
     	DBH_close(d);
 }
 
+void remove_from_history(char *dbh_file,char *path2save){
+    	GString *gs;
+	DBHashTable *d;
+	history_dbh_t *history_dbh;
+	int size;
+	
+	if (strlen(path2save) > 255) return;
+	if ((d=DBH_open(dbh_file))==NULL){
+		if ((d=DBH_create(dbh_file, 11))==NULL) return;
+	}
+	gs = g_string_new(path2save);
+    	sprintf((char *)DBH_KEY(d), "%10u", g_string_hash(gs));
+   	g_string_free(gs, TRUE);
+
+	history_dbh = (history_dbh_t *)DBH_DATA(d);
+	if (DBH_load(d)){
+		DBH_erase(d);
+	} else return;
+    	DBH_close(d);
+}
+
+
