@@ -82,11 +82,16 @@
 
 static GList *run_list = NULL;
 
-#if 0
+/* do this: start a long copy operation, while it is proceeding, press run button
+ * and execute gkrellm. When the copy procedure finishes, xftree goes off into
+ * full cpu usage. A backtrace reveals that it is cycling on a p_threads mutex.
+ * (Bug in gthreads?). No mutexes are knowingly used, so maybe it would have been 
+ * better to use posix threads... Who knows. For the meanwhile, disactivate use of
+ * gthreads: */
+
 #ifndef G_THREADS_IMPL_NONE
 #define G_THREADS_IMPL_NONE
 #endif
-#endif
 
 #ifndef G_THREADS_IMPL_NONE
 static void *hilo_func(void *d)
@@ -205,7 +210,6 @@ int  parse_runline(char **argv,char *cmd_line,
 		gchar **app, gchar **arg){
     char *p;
     int i=0;
-    int argument_count;
     gboolean more_args,is_argv0=TRUE;
     /* remove leading and trailing whitespace */
     p=g_strstrip(cmd_line);
@@ -267,11 +271,10 @@ int  parse_runline(char **argv,char *cmd_line,
 int on_run(GtkTreeView * treeview, char *in_cmd, tree_entry_t * en, gboolean in_terminal, gboolean remember, gboolean put_in_history)
 {
     static gchar *gs=NULL;
-    gchar *p,*term=NULL;
-    GList *ee = run_list;
+    gchar *term=NULL;
+/*    GList *ee = run_list;*/
     int i,j;
     char *argv[MAX_ARGS], *cmd_line=NULL;
-    gboolean more_args,is_argv0=TRUE;
     gchar *app=NULL, *arg=NULL;
 
 #ifdef DEBUG
@@ -483,7 +486,6 @@ void set_reg_combo(GtkTreeView * treeview)
 
 void double_click_run(tree_details_t *tree_details,
 		tree_entry_t *en){
-    int i = 0;
     char *cmd;
     FILE *pipe;
     char *a=NULL;
