@@ -61,6 +61,8 @@
 #include "settings.h"
 
 #include "preferences.i"
+/* keep gcc happy */
+preferences_t *prefs = opt_branches_titles;
 
 #define ROOT	"xffm"
 
@@ -144,6 +146,7 @@ char **find_themes(char *where)
 -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/
 static xmlDocPtr xmlconfig = NULL;
 
+#if 0
 static xmlDocPtr make_empty_doc(void)
 {
     xmlDocPtr doc;
@@ -163,8 +166,7 @@ static xmlDocPtr make_empty_doc(void)
 
     return doc;
 }
-
-
+#endif
 
 void get_config(tree_details_t ** tree_details_p,char *rcfile)
 {
