@@ -44,6 +44,7 @@
 #include "apps.h"
 #include "add_file.h"
 #include "add_folder.h"
+#include "add_node_contents.h"
 #include "aux.h"
 #include "bookmarks.h"
 #include "dummies.h"
@@ -367,8 +368,13 @@ void open_dir(GtkTreeView * treeview, GtkTreeIter * iter, GtkTreePath * treepath
 	set_title(treeview,window_title);
     }
 
-    if(tree_details->preferences & AUTOSCROLL)
+    if(tree_details->preferences & AUTOSCROLL){
+	gdk_flush();
+	/*XXX gtk bug, scrolling does not always occur...
+	 * bugzilla bug report 120269
+	 * printf("DBG-a:scrolling...\n");*/
 	gtk_tree_view_scroll_to_cell(treeview, treepath, NULL, TRUE, 0.0, 0.0);
+    }
 
     cursor_reset(treeview);
     
@@ -655,7 +661,7 @@ gboolean treeclick(GtkWidget * widget, GdkEventButton * event, gpointer data)
 
 		else if(IS_EXE(en->type) )
 		{	/* run it (if not registered)*/
-      			reg_t *prg = reg_prog_by_file (en->path);
+			reg_t *prg = reg_prog_by_file (en->path);
 			if (prg) double_click_open_with(tree_details,en);
 			else {
     				/*  assume in_term to be safe */
@@ -766,7 +772,7 @@ void on_select_all_activate(GtkMenuItem * menuitem, gpointer user_data)
     GtkTreeView *treeview = get_selected_treeview((GtkWidget *) menuitem);
     GtkTreeSelection *selection = gtk_tree_view_get_selection(treeview);
     GtkTreeModel *treemodel = gtk_tree_view_get_model(treeview);
-    GtkTreeIter iter,child,parent;
+    GtkTreeIter child,parent;
     tree_entry_t *en;
     /*gboolean result;
 
