@@ -68,6 +68,8 @@
 #include "treeview.h"
 #include "treestore.h"
 #include "widgets.h"
+#include "smb/smb_misc.h"
+#include "libs/input.h"
 
 #include "preferences.i"
 
@@ -77,9 +79,12 @@
 
 #define LOCAL_OVERRIDE_MASK (SHOW_MM|SHOW_TB1|SHOW_TB2|SHOW_F|SHOW_TITLES|RSH_X_SSH|RSYNC_X_SCP)
 
+
 extern int geometryX, geometryY, hpane;
 extern char *auto_C_name[];
 extern char *auto_CC_name[];
+extern char *auto_D_name[];
+extern char *auto_DD_name[];
 extern gchar *bookfile;
 extern GList *theme_list;
 extern gchar *xffm_argv0;
@@ -136,11 +141,21 @@ static void show_alt_treeview(GtkWidget * parent)
     if (gtk_paned_get_position((GtkPaned *)vpaned) < vpaned->allocation.height * 0.40)
        gtk_paned_set_position(GTK_PANED(vpaned), vpaned->allocation.height * 0.40);
 }
+static void hide_alt_treeview(GtkWidget * parent)
+{
+    GtkWidget *vpaned;
+    if(!parent)
+	return;
+    /*printf("DBG:show_text\n");*/
+    vpaned = lookup_widget(parent, "hpaned1");
+    gtk_paned_set_position(GTK_PANED(vpaned), 0);
+}
 
 static void status_the_path(GtkTreeView *treeview, GtkTreePath *ipath,tree_entry_t *en)
 	    {
 		gchar *readable_path;
-    		GtkTreeModel *treemodel = gtk_tree_view_get_model(treeview);
+    		/*GtkTreeModel *treemodel =
+		 * gtk_tree_view_get_model(treeview);*/
     		tree_details_t *tree_details = get_tree_details(treeview);
 		
 		
@@ -212,9 +227,13 @@ static void add_autotype_C_widgets(GtkTreeView * treeview){
     create_menu_entry(auto_C_name[i]," ",open1_menu,tree_details->window,
 		    (void *)on_autotype_C,(gpointer)((long)i));
  }
+ for (i=0;auto_D_name[i];i++){
+    create_menu_entry(auto_D_name[i]," ",open1_menu,tree_details->window,
+		    (void *)on_autotype_D,(gpointer)((long)i));
+ }
   
-  create_menu_entry("autotype_DD","DD",open2_menu,tree_details->window,
-		    (void *)on_autotype_D,NULL);
+/*  create_menu_entry("autotype_DD","DD",open2_menu,tree_details->window,
+		    (void *)on_autotype_D,NULL);*/
   create_menu_entry("autotype_RR","RR",open2_menu,tree_details->window,
 		    (void *)on_autotype_R,NULL);
   
@@ -222,6 +241,10 @@ static void add_autotype_C_widgets(GtkTreeView * treeview){
       create_menu_entry(auto_CC_name[i],"CC",open2_menu,tree_details->window,
 		    (void *)on_autotype_C,(gpointer)((long)i));
   }
+  for (i=0;auto_DD_name[i];i++){
+      create_menu_entry(auto_DD_name[i],"DD",open2_menu,tree_details->window,
+		    (void *)on_autotype_D,(gpointer)((long)i));
+  }
 }
 
 static void add_preferences_items(GtkTreeView * treeview)
@@ -527,7 +550,7 @@ static gboolean treeview_key(GtkWidget *w,
     	GtkTreeModel *treemodel = gtk_tree_view_get_model(treeview);
     	tree_details_t *tree_details = get_tree_details(treeview);
 	    GtkTreePath *ipath=NULL;
-	GtkTreeIter iter,p_iter;
+	GtkTreeIter iter,iter2;
 	static gboolean lock=FALSE;
         tree_entry_t *en=NULL;
 	if (tree_details->loading){
@@ -608,13 +631,19 @@ static gboolean treeview_key(GtkWidget *w,
 			GtkTreeView *treeview2;
 			lock=TRUE;
 			on_unselect_activate((GtkMenuItem *)treeview,NULL);
-			if (tree_details->treeview==treeview){
+			if ((event->state&GDK_CONTROL_MASK)){
+				treeview2=tree_details->treeview;
+				ref=&Rref;
+				hide_alt_treeview((GtkWidget *)treeview);
+			} else {	
+			  if (tree_details->treeview==treeview){
 				ref=&Lref;
 				treeview2=(GtkTreeView *)lookup_widget((GtkWidget *)treeview, "treeview2");
 				show_alt_treeview((GtkWidget *)treeview);
-			} else {
+			  } else {
 				treeview2=tree_details->treeview;
 				ref=&Rref;
+			  }
 			}
 			selection2 = gtk_tree_view_get_selection(treeview2);
 
@@ -678,16 +707,39 @@ static gboolean treeview_key(GtkWidget *w,
   		   gtk_tree_model_get(treemodel, &iter, ENTRY_COLUMN, &en, -1);
 	gtk_tree_selection_set_mode(selection, GTK_SELECTION_MULTIPLE);
 
-	if ((event->keyval >= GDK_A && event->keyval <= GDK_Z) || 
-	    (event->keyval >= GDK_a && event->keyval <= GDK_z) )
+	if ((event->keyval >= GDK_space && event->keyval <= GDK_asciitilde) || 
+	    (event->keyval >= GDK_KP_Multiply && event->keyval <= GDK_KP_9)  
+	    )
 	{
 	    char c;
+	    gchar *f;
 	    int emergency=0;
+	    gboolean second_letter_match=FALSE;
 	    gchar *a=NULL,*b=NULL;
-	    if (event->keyval >= GDK_a && event->keyval <= GDK_z)
-		c=event->keyval - GDK_a + 'a';
-	    else
-		c=event->keyval - GDK_A + 'A';
+	    if (event->keyval >= GDK_space && event->keyval <= GDK_asciitilde)
+		c=event->keyval;
+	    else switch (event->keyval){
+		case GDK_KP_0: c=GDK_0; break;
+		case GDK_KP_1: c=GDK_1; break;
+		case GDK_KP_2: c=GDK_2; break;
+		case GDK_KP_3: c=GDK_3; break;
+		case GDK_KP_4: c=GDK_4; break;
+		case GDK_KP_5: c=GDK_5; break;
+		case GDK_KP_6: c=GDK_6; break;
+		case GDK_KP_7: c=GDK_7; break;
+		case GDK_KP_8: c=GDK_8; break;
+		case GDK_KP_9: c=GDK_9; break;
+		case GDK_KP_Multiply: c=GDK_asterisk; break;
+		case GDK_KP_Add: c=GDK_plus; break;
+		case GDK_KP_Separator: c=GDK_space; break;
+		case GDK_KP_Subtract: c=GDK_minus; break;
+		case GDK_KP_Decimal: c=GDK_period; break;
+		case GDK_KP_Divide: c=GDK_slash; break;
+		default : c=0x7f; break;
+	    }
+	    /*printf ("state=0x%x\n",event->state);*/
+	    /* Use windows key modifier to do second letter match */
+	    if (event->state & GDK_MOD4_MASK) second_letter_match=TRUE;
 
 #ifdef DEBUG	
 	    printf("DBG: alphabeta keyval=0x%x\n",event->keyval);
@@ -695,37 +747,58 @@ static gboolean treeview_key(GtkWidget *w,
 	    if (!en) {
 		if (!gtk_tree_model_get_iter_first (treemodel,&iter))
 		    return FALSE;
-	    } 
+	    } 	    
+	    if (ipath) gtk_tree_path_free(ipath);	
+	    ipath=gtk_tree_model_get_path(treemodel, &iter);
 	    
-            ipath=gtk_tree_model_get_path(treemodel, &iter);
 	    a=gtk_tree_path_to_string(ipath);
-    
+    	    gtk_tree_model_get(treemodel, &iter, NAME_COLUMN, &f, ENTRY_COLUMN, &en, -1);
+#ifdef DEBUG	
+	    printf("DBG: startpath=%s\n",a);
+#endif	    
 	    do {
-		gchar *f;
-		GtkTreeIter iter2;
+		GtkTreeIter parent;
+		if (!gtk_tree_model_get_iter (treemodel,&iter,ipath)) {
+			printf("xffm:treeview.c this is unreachable (728)\n");
+			break;
+		}
+			
 		if (gtk_tree_view_row_expanded (treeview,ipath)){
 			gtk_tree_path_down (ipath);
-			if (!gtk_tree_model_get_iter (treemodel,&iter2,ipath))
-				g_assert_not_reached();
+			if (!gtk_tree_model_get_iter (treemodel,&iter,ipath)){
+			    printf("xffm:treeview.c this is unreachable (735)\n");
+			    break;
+			}			
 		} else {
 		     gtk_tree_path_next (ipath);
-		     if (!gtk_tree_model_get_iter (treemodel,&iter2,ipath)){
+		     if (gtk_tree_model_get_iter (treemodel,&iter2,ipath)) iter=iter2;
+		     else {
 			if (gtk_tree_model_iter_parent (treemodel,&iter2,&iter)){
-			  if (!gtk_tree_model_iter_next (treemodel,&iter2)){
+			  if (gtk_tree_model_iter_next (treemodel,&iter2))iter=iter2;
+			  else {
 		              gtk_tree_model_get_iter_first (treemodel,&iter);
-			  } else iter=iter2;
-			} else gtk_tree_model_get_iter_first (treemodel,&iter);
-			if (ipath) gtk_tree_path_free(ipath);
-			iter2=iter;
-			ipath=gtk_tree_model_get_path(treemodel, &iter2);
+			  } 
+			} else {
+				gtk_tree_model_get_iter_first (treemodel,&iter);
+			}
 		     }
 		}
-	
-    		gtk_tree_model_get(treemodel, &iter2, NAME_COLUMN, &f, ENTRY_COLUMN, &en, -1);
+		
+		if (ipath) gtk_tree_path_free(ipath);	
+		ipath=gtk_tree_model_get_path(treemodel, &iter);
+		
+    		gtk_tree_model_get(treemodel, &iter, NAME_COLUMN, &f, ENTRY_COLUMN, &en, -1);
 		emergency++;
-		if (!f) continue;
-		/*printf("DBG:(%s) %d==%d\n",en->path,*f,c);*/
+		if (!f || !strlen(f)) {
+#ifdef DEBUG	
+	    		printf("DBG: treemodel_get returns null for name column cell!\n");
+#endif			
+			continue;
+		}
+		/*printf("DBG:(%s,%s) %d==%d ... emergency=%d\n",en->path,f,*f,c,emergency);fflush(NULL);*/
+		if (second_letter_match && strlen(f)>1) f++;
 		if (*f == c){
+			gdk_flush();
     			gtk_tree_view_scroll_to_cell(treeview, ipath, 
 					NULL, TRUE, 0.0, 0.0);
 			gtk_tree_selection_select_path(selection,ipath);
@@ -736,14 +809,18 @@ static gboolean treeview_key(GtkWidget *w,
 		} 
 		g_free(b);
 		b=gtk_tree_path_to_string(ipath);
-	    } while (strcmp(a,b) || emergency<13000);
-	    g_free(a);
-	    g_free(b);
-	    if (emergency >= 13000){
-		    /* this should probably never happen */
-		    print_diagnostics(treeview,"xf_ERROR_ICON",
+#ifdef DEBUG	
+	    printf("DBG: skipping to path=%s\n",b);
+#endif	
+	    } while (strcmp(a,b) && emergency<20000);
+	    if (emergency >= 20000){
+		    /* this should never happen */
+		    print_diagnostics(treeview,"xf_ERROR_ICON", 
+			"unreachable code reached! (treeview.c 790) ", 
 			strerror(ETIMEDOUT),"\n",NULL);
 	    }
+	    g_free(a);
+	    g_free(b);
 	    
     	    if (ipath) gtk_tree_path_free(ipath);
 	    return FALSE;
@@ -758,6 +835,10 @@ static gboolean treeview_key(GtkWidget *w,
 			 tb_find((GtkButton *)w,NULL);
 			 return TRUE;
 		case GDK_Down:
+#ifdef DEBUG
+		     printf("DBG:gdk down...state=0x%x, mod3 is 0x%x\n",
+				     event->state,GDK_MOD3_MASK);
+#endif
 	        case GDK_Up:
 			if (event->state&GDK_SHIFT_MASK) return FALSE;
 		         lock=TRUE;
@@ -806,7 +887,13 @@ static gboolean treeview_key(GtkWidget *w,
 			gtk_tree_model_get_iter(treemodel, &iter, ipath);
 			gtk_tree_model_get(treemodel, &iter, ENTRY_COLUMN, &en, -1);
 
-			if (en) status_the_path(treeview,ipath,en);	
+			if (en) status_the_path(treeview,ipath,en);
+			/* scrolling with alt key */	
+			if (event->state&GDK_MOD1_MASK){
+				/*get selected path ++*/
+			     gdk_flush();
+			     gtk_tree_view_scroll_to_cell(treeview, ipath, NULL, TRUE, 0.0, 0.0);
+			}
 			
 			gtk_tree_path_free(ipath);
 		        lock=FALSE;
@@ -882,7 +969,6 @@ static gboolean treeview_key(GtkWidget *w,
 			if (gtk_tree_model_iter_has_child(treemodel,&iter))
 			{
     			     GtkTreePath *treepath;
-			     GtkTreeIter parent;
 			     treepath = gtk_tree_model_get_path(treemodel, &iter);
 			     gtk_tree_view_collapse_row(treeview, treepath);
 		    	     gtk_tree_path_free(treepath);
@@ -1035,14 +1121,7 @@ GtkTreeView *create_treeview(char *path)
     /* create the label column */
     name_column = column = gtk_tree_view_column_new();
     gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
-    /* set resize to FALSE to avoid name column from growing
-     * with icon changes. But a bug in gtk (2.0.6 at least)
-     * causes the tree to flash after scrolling and opening
-     * or closing a node. Therefore we stay with TRUE (less annoying bug)
-     * until the bug is fixed (bugzilla bug#116939)
-     * XXX Is it still happening in gtk 2.2?
-     * */
-    gtk_tree_view_column_set_resizable(column, TRUE);
+    gtk_tree_view_column_set_resizable(column, FALSE);
     gtk_tree_view_column_set_reorderable(column, TRUE);
     gtk_tree_view_column_set_spacing(column,2);
 
@@ -1388,13 +1467,14 @@ GtkTreeView *init_xffm(int argc, char *argv[])
 #ifdef DEBUG
     printf("DBG:xffm_argv0=%s\n",xffm_argv0);
 #endif
-    if (strstr(xffm_argv0,"xfsamba") ||
-	strstr(xffm_argv0,"xfbook") ||    
-	strstr(xffm_argv0,"xftrash") ||    
-	strstr(xffm_argv0,"xfapps") ||    
-	strstr(xffm_argv0,"xftree") ||    
-	strstr(xffm_argv0,"xfglob") ||    
-	strstr(xffm_argv0,"xffstab") && argc < 2 ) 
+    if ((strstr(xffm_argv0,"xfsamba") ||
+	 strstr(xffm_argv0,"xfbook") ||    
+	 strstr(xffm_argv0,"xftrash") ||    
+	 strstr(xffm_argv0,"xfapps") ||    
+	 strstr(xffm_argv0,"xftree") ||    
+	 strstr(xffm_argv0,"xfglob") ||    
+	 strstr(xffm_argv0,"xffstab") ) && 
+	argc < 2 ) 
 	    expand=TRUE;
     if((strstr(xffm_argv0,"xffm") || strstr(xffm_argv0,"xftree"))&& argc >= 2)
     {				/* if it fails it goes to root. */
@@ -1438,7 +1518,8 @@ GtkTreeView *init_xffm(int argc, char *argv[])
            GtkTreeModel *treemodel = gtk_tree_view_get_model(treeview);
            GtkTreePath *treepath = gtk_tree_model_get_path(treemodel, &iter);
     	   gtk_tree_view_expand_row(treeview, treepath, FALSE);  
-     	   gtk_tree_view_scroll_to_cell(treeview, treepath, NULL, TRUE, 0.0, 0.0);
+     	   gdk_flush();
+	   gtk_tree_view_scroll_to_cell(treeview, treepath, NULL, TRUE, 0.0, 0.0);
 	   gtk_tree_selection_select_path (selection,treepath);
 	   gtk_tree_view_set_cursor (treeview,treepath,NULL,FALSE);
  	   gtk_tree_path_free(treepath); 
