@@ -1258,7 +1258,7 @@ create_menu (GtkWidget * top)
 			     (gpointer) cb_patchapply, (gpointer) ((int) 1));
 
   /* Create "Help" menu */
-  menu = shortcut_menu (RIGHT_MENU, menubar, _("Help"), NULL, NULL);
+  menu = shortcut_menu (MENUBAR, menubar, _("Help"), NULL, NULL);
   shortcut_menu (SUBMENU, menu, _("About"), (gpointer) cb_about, NULL);
 
   /* hide and show whatever */
